@@ -27,7 +27,7 @@ from oslo_utils import reflection
 from oslo_utils import timeutils
 import six
 
-from oslo_service._i18n import _LE, _LW, _
+from oslo_service._i18n import _
 
 LOG = logging.getLogger(__name__)
 
@@ -67,7 +67,7 @@ def _safe_wrapper(f, kind, func_name):
         except LoopingCallDone:
             raise  # let the outer handler process this
         except Exception:
-            LOG.error(_LE('%(kind)s %(func_name)r failed'),
+            LOG.error('%(kind)s %(func_name)r failed',
                       {'kind': kind, 'func_name': func_name},
                       exc_info=True)
             return 0
@@ -149,7 +149,7 @@ class LoopingCallBase(object):
         except Exception:
             exc_info = sys.exc_info()
             try:
-                LOG.error(_LE('%(kind)s %(func_name)r failed'),
+                LOG.error('%(kind)s %(func_name)r failed',
                           {'kind': kind, 'func_name': func_name},
                           exc_info=exc_info)
                 self.done.send_exception(*exc_info)
@@ -173,8 +173,8 @@ class FixedIntervalLoopingCall(LoopingCallBase):
             delay = round(elapsed - interval, 2)
             if delay > 0:
                 func_name = reflection.get_callable_name(self.f)
-                LOG.warning(_LW('Function %(func_name)r run outlasted '
-                                'interval by %(delay).2f sec'),
+                LOG.warning('Function %(func_name)r run outlasted '
+                            'interval by %(delay).2f sec',
                             {'func_name': func_name, 'delay': delay})
             return -delay if delay < 0 else 0
         return self._start(_idle_for, initial_delay=initial_delay,
@@ -198,8 +198,8 @@ class FixedIntervalWithTimeoutLoopingCall(LoopingCallBase):
             delay = round(elapsed - interval, 2)
             if delay > 0:
                 func_name = reflection.get_callable_name(self.f)
-                LOG.warning(_LW('Function %(func_name)r run outlasted '
-                                'interval by %(delay).2f sec'),
+                LOG.warning('Function %(func_name)r run outlasted '
+                            'interval by %(delay).2f sec',
                             {'func_name': func_name, 'delay': delay})
             elapsed_time = time.time() - start_time
             if timeout > 0 and elapsed_time > timeout:
@@ -290,6 +290,8 @@ class BackOffLoopingCall(LoopingCallBase):
                    random.gauss(jitter, 0.1), with jitter as the mean for the
                    distribution. If set below .5, it can cause the calls to
                    come more rapidly after each failure.
+    :param min_interval: The minimum interval in seconds between calls to
+                         function.
     :raises: LoopingCallTimeout if time spent doing error retries would exceed
              timeout.
     """
@@ -305,7 +307,7 @@ class BackOffLoopingCall(LoopingCallBase):
         self._interval = 1
 
     def start(self, initial_delay=None, starting_interval=1, timeout=300,
-              max_interval=300, jitter=0.75):
+              max_interval=300, jitter=0.75, min_interval=0.001):
         if self._thread is not None:
             raise RuntimeError(self._RUN_ONLY_ONE_MESSAGE)
 
@@ -314,16 +316,20 @@ class BackOffLoopingCall(LoopingCallBase):
         self._interval = starting_interval
 
         def _idle_for(success, _elapsed):
-            random_jitter = self._RNG.gauss(jitter, 0.1)
+            random_jitter = abs(self._RNG.gauss(jitter, 0.1))
             if success:
                 # Reset error state now that it didn't error...
                 self._interval = starting_interval
                 self._error_time = 0
                 return self._interval * random_jitter
             else:
-                # Perform backoff
-                self._interval = idle = min(
-                    self._interval * 2 * random_jitter, max_interval)
+                # Perform backoff, random jitter around the next interval
+                # bounded by min_interval and max_interval.
+                idle = max(self._interval * 2 * random_jitter, min_interval)
+                idle = min(idle, max_interval)
+                # Calculate the next interval based on the mean, so that the
+                # backoff grows at the desired rate.
+                self._interval = max(self._interval * 2 * jitter, min_interval)
                 # Don't go over timeout, end early if necessary. If
                 # timeout is 0, keep going.
                 if timeout > 0 and self._error_time + idle > timeout:
